from abc import ABC, abstractmethod
from typing import List


class AuthApiMixin(ABC):

    @abstractmethod
    def request(self, uri, method, body_params=None, query_params=None, required_auth_token=False):
        pass

    def get_apikeys(self) -> List[str]:
        """
        Get api-keys of authenticated user
        :return List with api-keys of authenticated user:
        """
        uri = 'users/apiKeys'
        return self.request(uri, method='GET', required_auth_token=True)


import json
import os
import signal
import sys
from datetime import datetime
from typing import List

from botocore.exceptions import ClientError
from warrant import Cognito
from superai.config import BASE_FOLDER, COGNITO_USERPOOL_ID, COGNITO_CLIENT_ID, COGNITO_REGION
from superai.client import Client
import click


def _signal_handler(s, f):
    sys.exit(1)


def save_api_key(api_key: str):
    api_key_file = os.path.expanduser(f'{BASE_FOLDER}/apikey')
    if not os.path.exists(os.path.dirname(api_key_file)):
        os.makedirs(os.path.dirname(api_key_file))
    with open(api_key_file, 'w') as f:
        f.write(api_key)


@click.group()
def cli():
    pass


def load_api_key() -> str:
    api_key_file = os.path.expanduser(f'{BASE_FOLDER}/apikey')
    with open(api_key_file) as f:
        api_key = f.readline()
    return api_key


@cli.group()
@click.pass_context
def client(ctx):
    """
    super.AI API operations
    """
    api_key = ''
    try:
        api_key = load_api_key()
    except Exception as e:
        pass
    if len(api_key) == 0:
        print('User needs to login or set api key')
        exit()
    ctx.obj = {}
    ctx.obj['client'] = Client(api_key=api_key)


@client.command(name='create_jobs')
@click.option('--app_id', '-a', help='Application id', required=True)
@click.option('--callback_url', '-c', help='Callback URL for post when jobs finish')
@click.option('--inputs', '-i', help='Json list with inputs')
@click.option('--inputs_file', '-if', help='URL pointing to JSON file')
@click.pass_context
def create_jobs(ctx, app_id: str, callback_url: str, inputs: str, inputs_file: str):
    """
    Submit jobs
    """
    client = ctx.obj['client']
    print("Submitting jobs")
    json_inputs = None
    if inputs is not None:
        try:
            json_inputs = json.loads(inputs)
        except:
            print("Couldn't read json inputs")
            exit()
    print(client.create_jobs(app_id, callback_url, json_inputs, inputs_file))


@client.command(name='fetch_job')
@click.option('--job_id', '-j', help='Job id', required=True)
@click.pass_context
def fetch_job(ctx, job_id: str):
    """
    Get Job given job id
    """
    client = ctx.obj['client']
    print(f'Fetching job {job_id}')
    print(client.fetch_job(job_id))

@client.command(name='fetch_batches_job')
@click.option('--app_id', '-a', help='App id', required=True)
@click.pass_context
def fetch_batches_job(ctx, app_id: str):
    """
    Get not processed Batches given app id
    """
    client = ctx.obj['client']
    print(f'Fetching batches {app_id}')
    print(client.fetch_batches_job(app_id))

@client.command(name='fetch_batch_job')
@click.option('--app_id', '-a', help='App id', required=True)
@click.option('--batch_id', '-b', help='Batch id', required=True)
@click.pass_context
def fetch_batch_job(ctx, app_id: str, batch_id: str):
    """
    Get Batch given app id and batch id
    """
    client = ctx.obj['client']
    print(f'Fetching batch {app_id} {batch_id}')
    print(client.fetch_batch_job(app_id, batch_id))


@client.command(name='get_job_response')
@click.option('--job_id', '-j', help='Job id', required=True)
@click.pass_context
def get_job_response(ctx, job_id: str):
    """
    Get Job response given job id
    """
    client = ctx.obj['client']
    print(f'Getting job response {job_id}')
    print(client.get_job_response(job_id))


@client.command(name='cancel_job')
@click.option('--job_id', '-j', help='Job id', required=True)
@click.pass_context
def cancel_job(ctx, job_id: str):
    """
    Cancel a job given job id. Only for jobs in SCHEDULED, IN_PROGRESS or SUSPENDED state.
    """
    client = ctx.obj['client']
    print(f'Cancelling job {job_id}')
    print(client.cancel_job(job_id))


@client.command(name='list_jobs')
@click.option('--app_id', '-a', help='Application id', required=True)
@click.option('--page', '-p', help='Page number', type=int)
@click.option('--size', '-s', help='Size of page', type=int)
@click.option('--sort_by', '-sort', help='Job field to sort by', type=str, default='id', show_default=True)
@click.option('--order_by', '-order', help='Sort direction (asc or desc)',
              type=click.Choice(['asc', 'desc']), default='asc', show_default=True)
@click.option('--created_start_date', '-c0', help='Created start date',
              type=click.DateTime(formats=['%Y-%m-%dT%H:%M:%SZ', '%Y-%m-%d']))
@click.option('--created_end_date', '-c1', help='Created end date',
              type=click.DateTime(formats=['%Y-%m-%dT%H:%M:%SZ', '%Y-%m-%d']))
@click.option('--completed_start_date', '-e0', help='Completed start date',
              type=click.DateTime(formats=['%Y-%m-%dT%H:%M:%SZ', '%Y-%m-%d']))
@click.option('--completed_end_date', '-e1', help='Completed end date',
              type=click.DateTime(formats=['%Y-%m-%dT%H:%M:%SZ', '%Y-%m-%d']))
@click.option('--status_in', '-s_in', help='Status of jobs', multiple=True, type=click.Choice(
    ['SCHEDULED', 'IN_PROGRESS', 'FAILED', 'SUSPENDED', 'CANCELED', 'EXPIRED', 'COMPLETED']))
@click.pass_context
def list_jobs(ctx, app_id: str, page: int, size: int, sort_by: str, order_by: str, created_start_date: datetime,
              created_end_date: datetime, completed_start_date: datetime, completed_end_date: datetime,
              status_in: List[str] = None):
    """
    Get a paginated list of jobs (without response) given an application id
    """
    client = ctx.obj['client']
    print(f'Fetching jobs per application {app_id}')
    if len(status_in) == 0:
        status_in = None
    print(client.list_jobs(app_id, page, size, sort_by, order_by, created_start_date, created_end_date,
                           completed_start_date,
                           completed_end_date, status_in))


@client.command(name='download_jobs')
@click.option('--app_id', '-a', help='Application id', required=True)
@click.option('--created_start_date', '-c0', help='Created start date',
              type=click.DateTime(formats=['%Y-%m-%dT%H:%M:%SZ', '%Y-%m-%d']))
@click.option('--created_end_date', '-c1', help='Created end date',
              type=click.DateTime(formats=['%Y-%m-%dT%H:%M:%SZ', '%Y-%m-%d']))
@click.option('--completed_start_date', '-e0', help='Completed start date',
              type=click.DateTime(formats=['%Y-%m-%dT%H:%M:%SZ', '%Y-%m-%d']))
@click.option('--completed_end_date', '-e1', help='Completed end date',
              type=click.DateTime(formats=['%Y-%m-%dT%H:%M:%SZ', '%Y-%m-%d']))
@click.option('--status_in', '-s_in', help='Status of jobs', multiple=True, type=click.Choice(
    ['SCHEDULED', 'IN_PROGRESS', 'FAILED', 'SUSPENDED', 'CANCELED', 'EXPIRED', 'COMPLETED']))
@click.pass_context
def download_jobs(ctx, app_id: str, created_start_date: datetime,
                  created_end_date: datetime, completed_start_date: datetime, completed_end_date: datetime,
                  status_in: List[str] = None):
    """
    Trigger processing of job responses that is sent to customer email once is finished.
    """
    client = ctx.obj['client']
    print(f'Triggering job responses processing per application {app_id}')
    if len(status_in) == 0:
        status_in = None
    print(client.download_jobs(app_id, created_start_date, created_end_date, completed_start_date, completed_end_date,
                               status_in))


@client.command(name='create_ground_truth')
@click.option('--app_id', '-a', help='Application id', required=True)
@click.option('--input_json', '-i', help='Input json of ground truth', required=True)
@click.option('--label', '-l', help='Label (or output) json of ground truth', required=True)
@click.option('--tag', '-t', help='Tag ground truth data')
@click.pass_context
def create_ground_truth(ctx, app_id: str, input_json: dict = None, label: dict = None, tag: str = None):
    """
    Submit fresh ground truth data
    """
    client = ctx.obj['client']
    print("Submitting fresh ground truth data")
    print(client.create_ground_truth(app_id, input_json, label, tag))


@client.command(name='update_ground_truth')
@click.option('--ground_truth_data_id', '-g', help='Ground truth data id', required=True)
@click.option('--input_json', '-i', help='Input json of ground truth')
@click.option('--label', '-l', help='Label (or output) json of ground truth')
@click.option('--tag', '-t', help='Tag ground truth data')
@click.pass_context
def update_ground_truth(ctx, ground_truth_data_id: str, input_json: dict = None, label: dict = None, tag: str = None):
    """
    Update (patch) ground truth data
    """
    client = ctx.obj['client']
    print(f"Updating ground truth data {ground_truth_data_id}")
    print(client.update_ground_truth(ground_truth_data_id, input_json, label, tag))


@client.command(name='list_ground_truth_data')
@click.option('--app_id', '-a', help='Application id', required=True)
@click.option('--page', '-p', help='Page number', type=int)
@click.option('--size', '-s', help='Size of page', type=int)
@click.pass_context
def list_ground_truth_data(ctx, app_id: str, page: int, size: int):
    """
    List all ground truth data for an application
    """
    client = ctx.obj['client']
    print(f"Fetching ground truth data per application {app_id}")
    print(client.list_ground_truth_data(app_id, page, size))


@client.command(name='get_ground_truth_data')
@click.option('--ground_truth_data_id', '-g', help='Ground truth data id', required=True)
@click.pass_context
def get_ground_truth_data(ctx, ground_truth_data_id: str):
    """
    Fetch single ground truth data object
    """
    client = ctx.obj['client']
    print(f"Fetching ground truth data {ground_truth_data_id}")
    print(client.get_ground_truth_data(ground_truth_data_id))


@client.command(name='delete_ground_truth_data')
@click.option('--ground_truth_data_id', '-g', help='Ground truth data id', required=True)
@click.pass_context
def delete_ground_truth_data(ctx, ground_truth_data_id: str):
    """
    Mark ground truth data as deleted
    """
    client = ctx.obj['client']
    print(f"Deleting ground truth data {ground_truth_data_id}")
    print(client.delete_ground_truth_data(ground_truth_data_id))


@client.command(name='delete_ground_truth_data')
@click.option('--app_id', '-a', help='Application id', required=True)
@click.option('-job_id', '-j', help='Job id', required=True)
@click.pass_context
def create_ground_truth_from_job(ctx, app_id: str, job_id: str):
    client = ctx.obj['client']
    print(f"Converting job {job_id} to ground truth data")
    print(client.create_ground_truth_from_job(app_id, job_id))


@cli.command()
@click.option('--api-key', help='Your super.AI API KEY', required=True)
def config(api_key):
    """
    Set api key.
    """
    save_api_key(api_key)


@cli.command()
@click.option('--username', '-u', help='super.AI Username', required=True)
@click.option('--password', prompt=True, hide_input=True)
def login(username, password):
    """
    Use username and password to get super.AI api key.
    """
    user = Cognito(access_key='AKIAIOSFODNN7EXAMPLE', secret_key='wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY',
                   user_pool_id=COGNITO_USERPOOL_ID, client_id=COGNITO_CLIENT_ID,
                   user_pool_region=COGNITO_REGION, username=username)
    try:
        user.authenticate(password)
    except ClientError as e:
        if e.response['Error']['Code'] == 'UserNotFoundException' or e.response['Error'][
            'Code'] == 'NotAuthorizedException':
            print("Incorrect username or password")
            return
        else:
            print(f"Unexpected error: {e}")
            return

    client = Client(auth_token=user.access_token)
    api_keys = client.get_apikeys()
    if len(api_keys) > 0:
        save_api_key(api_keys[0])
        print(f'Api key {api_keys[0]} was set')
    else:
        print(f'User {username} doesn\'t have any api keys')


@cli.command()
def logout():
    """
    Remove stored api key
    """
    save_api_key('')
    print('Stored api key was removed')


def main():
    signal.signal(signal.SIGINT, _signal_handler)
    sys.exit(cli())


if __name__ == '__main__':
    main()



from abc import ABC, abstractmethod
from typing import List, BinaryIO, Generator
import requests

from superai.exceptions import SuperAIStorageError


class DataApiMixin(ABC):
    _resource = 'data'

    @abstractmethod
    def request(self, uri, method, body_params=None, query_params=None, required_api_key=False):
        pass

    @property
    def resource(self):
        return self._resource

    def list_data(self, data_ids: List[str] = None, paths: List[str] = None, recursive: bool = False,
                  signedUrl: bool = False, secondsTtl: int = 600, page: int = None, size: int = None) -> dict:
        """
        Get a paginated list of datasets, that can be filtered using an array of ids xor and array of paths
        :param data_ids: Array of data ids
        :param paths: Array of paths
        :param recursive: Get all datasets from recursive path (only takes first path of array)
        :param signedUrl: Get signed url for each dataset
        :param secondsTtl: Time to live for signed url
        :param page: Page number [0..N]
        :param size: Size of page
        :return: Paginated list of datasets dicts
        """
        query_params = {}
        if data_ids is not None:
            query_params['dataId'] = data_ids
        elif paths is not None:
            query_params['paths'] = paths
            query_params['recursive'] = recursive
        query_params['signedUrl'] = signedUrl
        if signedUrl:
            query_params['secondsTtl'] = secondsTtl
        if page is not None:
            query_params['page'] = page
        if size is not None:
            query_params['size'] = size
        return self.request(self.resource, method='GET', query_params=query_params, required_api_key=True)

    def get_all_data(self, data_ids: List[str] = None, paths: List[str] = None, recursive: bool = False,
                     signedUrl: bool = False, secondsTtl: int = 600) -> Generator[dict, None, None]:
        """
        Generator that retrieves all data filtered using an array of ids xor and array of paths
        :param data_ids: Array of data ids
        :param paths: Array of paths
        :param recursive: Get all datasets from recursive path (only takes first path of array)
        :param signedUrl: Get signed url for each dataset
        :param secondsTtl: Time to live for signed url
        :return: Generator that yields complete list of dicts with data objects
        """
        page = 0
        paginated_data = {'last': False}
        while not paginated_data['last']:
            paginated_data = self.list_data(data_ids=data_ids, paths=paths, recursive=recursive, signedUrl=signedUrl,
                                            secondsTtl=secondsTtl, page=page, size=500)
            for d in paginated_data['content']:
                yield d
            page = page + 1

    def get_signed_url(self, path: str, secondsTtl: int = 600) -> dict:
        """
        Get signed url for a dataset given its path
        :param path: Dataset's path
        :param secondsTtl: Time to live for signed url
        :return: Dict with signedUrl of dataset
        """
        uri = f'{self.resource}/url'
        return self.request(uri, method='GET', query_params={'path': path, 'secondsTtl': secondsTtl},
                            required_api_key=True)

    def delete_data(self, path: str) -> dict:
        """
        Delete dataset given its path
        :param path: Dataset's path
        :return: Dict with details of deleted dataset
        """
        return self.request(self.resource, method='DELETE', query_params={'path': path}, required_api_key=True)

    def upload_data(self, path: str, description: str, mimeType: str, file: BinaryIO) -> dict:
        """
        Create/update a dataset given its path using file and mimeType
        :param path: Path of dataset
        :param description: Description of dataset
        :param mimeType: Type of file
        :param file: Binary File value
        :return: Dataset created/updated
        """
        dataset = self.request(self.resource, method='POST',
                               query_params={'path': path, 'description': description, 'mimeType': mimeType,
                                             'uploadUrl': True}, required_api_key=True)
        try:
            resp = requests.put(dataset.pop('uploadUrl'), data=file.read())
            if resp.status_code == 200 or resp.status_code == 201:
                return dataset
            else:
                raise SuperAIStorageError(
                    f'File {str(file)} referenced by dataset {dataset["path"]} couldn\'t be uploaded to super.AI '
                    f'Storage')
        except Exception as e:
            raise SuperAIStorageError(
                f'File {str(file)} referenced by dataset {dataset["path"]} couldn\'t be uploaded to super.AI Storage '
                f'Error: {str(e)}')




class SuperAIError(Exception):

    def __init__(self, message: str, error_code: int):
        self.error_code = error_code
        self.message = message
        super(Exception, self).__init__(f'super.AI API returned {str(self.error_code)}: {self.message}')

class SuperAIStorageError(Exception):
    def __init__(self, message: str):
        self.message = message
        super(Exception, self).__init__(f'super.AI Storage service failed: {self.message}')


from abc import ABC, abstractmethod
from typing import Generator


class GroundTruthApiMixin(ABC):
    @abstractmethod
    def request(self, uri, method, body_params=None, query_params=None, required_api_key=False):
        pass

    def create_ground_truth(self, app_id: str, input_json: dict = None, label: dict = None, tag: str = None) -> dict:
        """
        Submit fresh ground truth data
        :param app_id: Application instance id
        :param input_json: Input that should match input schema of data program
        :param label: Label, or output that should match output schema of data program
        :param tag: Tag to identify ground truth data
        :return: Ground truth data object
        """
        body_json = {}
        if input_json is not None:
            body_json['input'] = input_json
        if label is not None:
            body_json['label'] = label
        if tag is not None:
            body_json['tag'] = tag
        uri = f'apis/{app_id}/baselineData'
        return self.request(uri, 'POST', body_params=body_json, required_api_key=True)

    def update_ground_truth(self, ground_truth_data_id: str, input_json: dict = None, label: dict = None,
                            tag: str = None) -> dict:
        """
        Upload (patch) ground truth data
        :param ground_truth_data_id: Id of ground truth data
        :param input_json: Input that should match input schema of data program
        :param label: Label, or output that should match output schema of data program
        :param tag: Tag to identify ground truth data
        :return: Updated ground truth data object
        """
        body_json = {}
        if input_json is not None:
            body_json['input'] = input_json
        if label is not None:
            body_json['label'] = label
        if tag is not None:
            body_json['tag'] = tag
        uri = f'baselineData/{ground_truth_data_id}'
        return self.request(uri, 'PATCH', body_params=body_json, required_api_key=True)

    def list_ground_truth_data(self, app_id: str, page: int = None, size: int = None) -> dict:
        """
        List all ground truth data for an application
        :param app_id: Application id
        :param page: Page number of results
        :param size: Page size of results
        :return: Paginated list of ground truth data objects
        """
        uri = f'apis/{app_id}/baselineData'
        q_params = {}
        if page is not None:
            q_params['page'] = page
        if size is not None:
            q_params['size'] = size
        return self.request(uri, 'GET', required_api_key=True, query_params=q_params)


    def get_all_ground_truth_data(self, app_id: str) -> Generator[dict, None, None]:
        """
        Generator that retrieves all ground truth data given an application id
        :param app_id: Application id
        :return: Generator that yields complete list of dicts with ground truth data objects
        """
        page = 0
        paginated_ground_truth = {'last': False}
        while not paginated_ground_truth['last']:
            paginated_ground_truth = self.list_ground_truth_data(app_id, page=page, size=500)
            for gtd in paginated_ground_truth['content']:
                yield gtd
            page = page + 1


    def get_ground_truth_data(self, ground_truth_data_id: str) -> dict:
        """
        Fetch single ground truth data object
        :param ground_truth_data_id: Id of ground truth data
        :return: Ground truth data object
        """
        uri = f'baselineData/{ground_truth_data_id}'
        return self.request(uri, 'GET', required_api_key=True)

    def delete_ground_truth_data(self, ground_truth_data_id) -> dict:
        """
        Mark ground truth data as deleted
        :param ground_truth_data_id: If of ground truth data
        :return Deleted ground truth daata object:
        """
        uri = f'baselineData/{ground_truth_data_id}'
        return self.request(uri, 'DELETE', required_api_key=True)

    def create_ground_truth_from_job(self, app_id: str, job_id: str) -> dict:
        """
        Convert completed job to ground truth data
        :param app_id: Application id
        :param job_id: Job id
        :return: Ground truth data object
        """
        uri = f'apis/{app_id}/baselineData/job/{job_id}'
        return self.request(uri, 'POST', required_api_key=True)



from datetime import datetime
from typing import List, Generator

from abc import ABC, abstractmethod


class JobsApiMixin(ABC):

    @abstractmethod
    def request(self, uri, method, body_params=None, query_params=None, required_api_key=False):
        pass

    def create_jobs(self, app_id: str, callbackUrl: str = None, inputs: List[dict] = None, inputsFileUrl: str = None,
                    metadata: dict = None) -> dict:
        """
        Submit jobs

        :param app_id: Application id
        :param callback_url: URL that should be POSTed once the job is completed for the response data.
        :param inputs: List of objects that represent the input of the job (based on the particular app type)
        :param inputsFileUrl: URL of json file containing list of input objects
        :param metadata: Object you can attach to job
        :return: Confirmation message of submission, batch id of submission, uuid of job if len(input) == 1
        """
        body_json = {}
        if callbackUrl is not None:
            body_json['callbackUrl'] = callbackUrl
        if inputs is not None:
            body_json['inputs'] = inputs
        if inputsFileUrl is not None:
            body_json['inputsFileUrl'] = inputsFileUrl
        if metadata is not None:
            body_json['metadata'] = metadata
        uri = f'apps/{app_id}/jobs'
        return self.request(uri, method='POST', body_params=body_json, required_api_key=True)

    def fetch_job(self, job_id: str) -> dict:
        """
        Get Job given job id

        :param job_id: Job id
        :return: Dict with job data
        """
        uri = f'jobs/{job_id}'
        return self.request(uri, method='GET', required_api_key=True)

    def fetch_batches_job(self, app_id) -> dict:
        """
        Get not processed Batches of submitted jobs given application id

        :param app_id: Application id
        :return: Dict with batch data
        """
        uri = f'apps/{app_id}/batches'
        return self.request(uri, method='GET', required_api_key=True)

    def fetch_batch_job(self, app_id: str, batch_id: str) -> dict:
        """
        Get Batch of submitted jobs given batch id and application id

        :param app_id: Application id
        :param batch_id: Batch id
        :return: Dict with batch data
        """
        uri = f'apps/{app_id}/batches/{batch_id}'
        return self.request(uri, method='GET', required_api_key=True)

    def get_job_response(self, job_id: str) -> dict:
        """
        Get Job Response given job id
        :param job_id:
        :return: Dict with job response
        """
        uri = f'jobs/{job_id}/response'
        return self.request(uri, method='GET', required_api_key=True)

    def cancel_job(self, job_id: str) -> dict:
        """
        Cancel a job given job id. Only for jobs in SCHEDULED, IN_PROGRESS or SUSPENDED state.

        :param job_id: Job id
        :return: Dict with job data
        """

        uri = f'jobs/{job_id}/cancel'
        return self.request(uri, method='POST', required_api_key=True)

    def list_jobs(self, app_id: str, page: int = None, size: int = None, sortBy: str = 'id', orderBy: str = 'asc',
                  createdStartDate: datetime = None, createdEndDate: datetime = None,
                  completedStartDate: datetime = None, completedEndDate: datetime = None,
                  statusIn: List[str] = None) -> dict:
        """
        Get a paginated list of jobs (without job responses) given an application id
        :param app_id: Application id
        :param page: Page number [0..N]
        :param size: Size of page
        :param sortBy: Job field to sort by
        :param orderBy: Sort direction (asc or desc)
        :param createdStartDate: Created start date
        :param createdEndDate: Created end date
        :param completedStartDate: Completed start date
        :param completedEndDate: Completed end date
        :param statusIn: Status of jobs
        :return: Paginated list of dicts with jobs data
        """
        uri = f'apps/{app_id}/jobs'
        query_params = {}
        if page is not None:
            query_params['page'] = page
        if size is not None:
            query_params['size'] = size
        if sortBy is not None:
            query_params['sortBy'] = sortBy
        if orderBy is not None:
            query_params['orderBy'] = orderBy
        if createdStartDate is not None:
            query_params['createdStartDate'] = createdStartDate.strftime('%Y-%m-%dT%H:%M:%SZ')
        if createdEndDate is not None:
            query_params['createdEndDate'] = createdEndDate.strftime('%Y-%m-%dT%H:%M:%SZ')
        if completedStartDate is not None:
            query_params['completedStartDate'] = completedStartDate.strftime('%Y-%m-%dT%H:%M:%SZ')
        if completedEndDate is not None:
            query_params['completedEndDate'] = completedEndDate.strftime('%Y-%m-%dT%H:%M:%SZ')
        if statusIn is not None:
            query_params['statusIn'] = statusIn
        return self.request(uri, method='GET', query_params=query_params, required_api_key=True)

    def download_jobs(self, app_id: str, createdStartDate: datetime = None, createdEndDate: datetime = None,
                      completedStartDate: datetime = None, completedEndDate: datetime = None,
                      statusIn: List[str] = None) -> str:
        """
        Trigger processing of jobs responses that is sent to customer email once is finished.
        :param app_id: Application id
        :param createdStartDate: Created start date
        :param createdEndDate: Created end date
        :param completedStartDate: Completed start date
        :param completedEndDate: Completed end date
        :param statusIn: Status of jobs
        :return: 'Task is processing' string
        """
        uri = f'apps/{app_id}/job_responses'
        query_params = {}
        if createdStartDate is not None:
            query_params['createdStartDate'] = createdStartDate.strftime('%Y-%m-%dT%H:%M:%SZ')
        if createdEndDate is not None:
            query_params['createdEndDate'] = createdEndDate.strftime('%Y-%m-%dT%H:%M:%SZ')
        if completedStartDate is not None:
            query_params['completedStartDate'] = completedStartDate.strftime('%Y-%m-%dT%H:%M:%SZ')
        if completedEndDate is not None:
            query_params['completedEndDate'] = completedEndDate.strftime('%Y-%m-%dT%H:%M:%SZ')
        if statusIn is not None:
            query_params['statusIn'] = statusIn
        return self.request(uri, method='POST', query_params=query_params, required_api_key=True)

    def get_all_jobs(self, app_id: str, sortBy: str = 'id', orderBy: str = 'asc', createdStartDate: datetime = None,
                     createdEndDate: datetime = None,
                     completedStartDate: datetime = None, completedEndDate: datetime = None,
                     statusIn: List[str] = None) -> Generator[dict, None, None]:
        """
        Generator that retrieves all jobs (without job responses) given an application id
        :param app_id: Application id
        :param sortBy: Job field to sort by
        :param orderBy: Sort direction (asc or desc)
        :param createdStartDate: Created start date
        :param createdEndDate: Created end date
        :param completedStartDate: Completed start date
        :param completedEndDate: Completed end date
        :param statusIn: Status of jobs
        :return: Generator that yields complete list of dicts with jobs data
        """
        page = 0
        paginated_jobs = {'pages': 1}
        while page <= paginated_jobs['pages'] - 1:
            paginated_jobs = self.list_jobs(app_id, page=page, size=500, sortBy=sortBy, orderBy=orderBy,
                                            createdStartDate=createdStartDate, createdEndDate=createdEndDate,
                                            completedStartDate=completedStartDate, completedEndDate=completedEndDate,
                                            statusIn=statusIn)
            for job in paginated_jobs['jobs']:
                yield job
            page = page + 1



# coding: utf-8


from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

NAME = "superai"
VERSION = "0.0.3"
# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

REQUIRES = [
    "boto3>=1.9",
    "requests>=2.22",
    "Click>=7.0",
    "warrant>=0.6",
]

setup(
    name=NAME,
    version=VERSION,
    description="super.AI API",
    author='super.AI',
    author_email="support@super.ai",
    url="https://github.com/mysuperai/superai-api-client",
    keywords=["super.AI API"],
    install_requires=REQUIRES,
    packages=find_packages(),
    include_package_data=True,
    long_description=long_description,
    long_description_content_type="text/markdown",
    entry_points={
        'console_scripts': [
            'superai=superai.cli:main',
        ],
    },
)



from typing import Optional

import requests

from superai.apis.auth import AuthApiMixin
from superai.apis.data import DataApiMixin
from superai.apis.jobs import JobsApiMixin
from superai.apis.ground_truth import GroundTruthApiMixin
from superai.config import BASE_URL
from superai.exceptions import SuperAIError


class Client(JobsApiMixin, AuthApiMixin, GroundTruthApiMixin, DataApiMixin):

    def __init__(self, api_key: str = None, auth_token: str = None, base_url: str = None):
        self.api_key = api_key
        self.auth_token = auth_token
        if base_url is None:
            self.base_url = BASE_URL
        else:
            self.base_url = base_url

    def request(self, endpoint: str, method: str = 'GET', query_params: dict = None, body_params: dict = None,
                required_api_key: bool = False, required_auth_token: bool = False) -> Optional[dict]:
        headers = {}
        if required_api_key and self.api_key:
            headers['API-KEY'] = self.api_key
        if required_auth_token and self.auth_token:
            headers['AUTH-TOKEN'] = self.auth_token
        resp = requests.request(method, f'{self.base_url}/{endpoint}', params=query_params, json=body_params,
                                headers=headers)
        try:
            resp.raise_for_status()
            if resp.status_code == 204:
                return None
            else:
                return resp.json()
        except requests.exceptions.HTTPError as http_e:
            try:
                message = http_e.response.json()['message']
            except:
                message = http_e.response.text
            raise SuperAIError(message, http_e.response.status_code)
